@@ -27,10 +27,13 @@ import inspect
 import glob
 import traceback
 import random
+import sqlite3
+
+import atexit
 
 import threading
 
-from datetime import datetime , timedelta
+from datetime import datetime , timedelta , timezone
 
 import datetime as _dt
 
@@ -62,6 +65,8 @@ class JsonEncoder( json.JSONEncoder ) :
         #    return float( obj)
         #elif isinstance( obj, numpy.ndarray ) :
         #    return obj.tolist( )
+        #if isinstance( obj , datetime.datetime ) :
+        #    return ( str( obj ) )
         if isinstance( obj , _dt.date ) :
             return str( obj ) 
         return json.JSONEncoder.default( self , obj )
@@ -95,7 +100,7 @@ def json_stdin( ) :
     return( data_in )
 
 def json_dumps( inputs ) :
-    return( json.dumps( inputs ) )
+    return( json.dumps( inputs , cls = JsonEncoder ) )
 
 #def json_pretty( json_in ) :
 #    data = json.loads(json_in)
@@ -103,18 +108,34 @@ def json_dumps( inputs ) :
 #    print(pretty_object)
 
 def json_prettyprint( data ) :
-    pretty_object = json.dumps( data , indent = 4 )
+    pretty_object = json.dumps( data , indent = 4 ,  cls = JsonEncoder )
     print( pretty_object )
 
 #####################################################################
 
+def db_getcon( db_fp ) :
+    db_con = sqlite3.connect( db_fp )
+
+#####################################################################
+
+def atexit_register( f ) :
+    atexit.register( f )
+
+def get_chunks(l, n):
+    """Yield n number of striped chunks from l."""
+    for i in range(0, n):
+        yield l[i::n]
+
+# Can be array
+def rnd_choice( i = "-|/\\"  ) :
+    return( random.choice( i ) )
 
 def get_ver( ) :
     return( get_version( ) )
 
 def num_randomlist( s = 10 ) :
-    random_list = [random.random() for i in range(s)]
-    return(random_list)
+    random_list = [ random.random( ) for i in range( s ) ]
+    return( random_list )
 
 # Get arg from position and use default
 def argv_get( p , d = None ) :
@@ -543,13 +564,35 @@ def display_time( seconds , granularity = 2 ) :
     return ', '.join( result[ :granularity ] )
 
 
+def datetime_strtotimestampms( dtstr ) :
+    return( datetime.strptime( dtstr , "%Y-%m-%dT%H:%M:%S" ).replace(tzinfo=timezone.utc).timestamp()*1000 )
+
+
+
+def datetime_istype( v ) :
+    return( type( v ) == datetime ) 
+
+
+def datetime_utcfromtimestamp( ts ) :
+    return( datetime.utcfromtimestamp( ts ).isoformat( ) )
+
+def datetime_isofromtimestamp( ts ) :
+    return(datetime.utcfromtimestamp( ts ).strftime( "%Y-%m-%dT%H:%M:%S" ))
+
+def datetime_isofromtimestampms( ts ) :
+    return(datetime.utcfromtimestamp( ts / 1000 ).strftime( "%Y-%m-%dT%H:%M:%S" ))
 
 def datetime_iso( ) :
     return( datetime.now( datetime.now( ).astimezone( ).tzinfo ).isoformat( ) )
 
+
+
 def time_now( ) :
     return( time.time( ) )
 
+def time_nowms( ) :
+    return( int( time.time( ) * 1000 ) )
+
 def time_now_str( ) :
     return( str( round( time_now( ) ) ) )
 
@@ -560,6 +603,9 @@ def datetime_utcnow( ) :
 def datetime_timestamp( ) :
     return( datetime.now( ).timestamp( ) )
 
+def datetime_utctimestamp( ) :
+    return( datetime.utcnow( ).timestamp( ) )
+
 def url_parse( u ) :
     return(urllib.parse.urlparse(u))
 
@@ -575,8 +621,8 @@ def idle( ) :
 
 def datetime_nowutcstr( td = 0 ) :
 
-    d = datetime.utcnow( ) + timedelta( seconds = td)
-
+    #d = datetime.now( timezone.utc) + timedelta( seconds = td )
+    d = datetime.utcnow( ) + timedelta( seconds = td )
     return( d.strftime( "%Y-%m-%dT%H:%M:%S" ) )
 
 
@@ -597,8 +643,8 @@ def datetime_timedelta_db( dbdtstr , td ) :
     return( d.strftime( "%Y-%m-%dT%H:%M:%S" ) )
 
 def datetime_inrange_db( dbdt1 , dbdt2 , td) :
-    d1 = datetime.strptime(dbdt1, '%Y-%m-%dT%H:%M:%S')  
-    d2 = datetime.strptime(dbdt2, '%Y-%m-%dT%H:%M:%S') 
+    d1 = datetime.strptime( dbdt1 , '%Y-%m-%dT%H:%M:%S' )  
+    d2 = datetime.strptime( dbdt2 , '%Y-%m-%dT%H:%M:%S' ) 
 
     dtdiff = d2 - d1
     dtdiff_seconds = int( dtdiff.total_seconds( ) )
@@ -621,7 +667,7 @@ def datetime_diffnow( dt2 ) :
 def datetime_centerwindow( p , ec , ew ):
     a = datetime_timedelta_db( p[ ec ] , -0.5 * p[ ew ] )
     b = datetime_timedelta_db( p[ ec ] , 0.5 * p[ ew ] )
-    return([a,b])
+    return( [ a , b ] )
 
 
 def datetime_initstartend( p , es , ee , ew ) :
@@ -676,11 +722,17 @@ def file_getcontent( fn , scp ) :
 
 
 def file_read(fp):
-    return(open( fp , "r" ).read( ))
+    return( open( fp , "r" ).read( ) )
 
 def file_exists( fp ) :
     return( os.path.isfile( fp ) )
 
+
+def prexit_fileexists( fp , pred = True ) :
+    if file_exists( fp ) == pred :
+        printexit( fp + " prexit_fileexists and PRED!")
+
+
 def file_ctime( fp ) :
     if(not file_exists(fp)): return(False)
     return( os.path.getctime( fp ) )
@@ -711,12 +763,15 @@ def sys_path_append_fp( fp ) :
 def file_stat( fp ) :
     return( os.stat( fp ) )
 
+def file_size( fp ) :
+    return( os.path.getsize( fp ) )
+
 def file_settimes( fp , t ) :
     os.utime( fp , ( t , t ) )
 
 
 def file_replace( fp_source , fp_target ):
-    os.replace(fp_source,fp_target)
+    os.replace( fp_source , fp_target )
 
 def file_tmprandom(  ) :
     fp = "/tmp/sapitmp_" + uhash( ) + ".tmp"
